@@ -1,26 +1,25 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 
 import os
 import subprocess
 import sys
-import urlparse
-import ConfigParser
+import configparser
 import aptsources.distro
 import aptsources.distinfo
 from aptsources.sourceslist import SourcesList
 import gettext
 import threading
 import pycurl
-import cStringIO
+from io import BytesIO
 from CountryInformation import CountryInformation
-import commands
 import re
 import json
 import datetime
-import urllib
-import pycurl
+from urllib.request import urlopen
+import requests
 from optparse import OptionParser
-from sets import Set
+import locale
+import mintcommon
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -32,6 +31,16 @@ BUTTON_LABEL_MAX_LENGTH = 30
 FLAG_PATH = "/usr/share/iso-flag-png/%s.png"
 FLAG_SIZE = 16
 
+# i18n
+APP = 'mintsources'
+LOCALE_DIR = "/usr/share/linuxmint/locale"
+locale.bindtextdomain(APP, LOCALE_DIR)
+gettext.bindtextdomain(APP, LOCALE_DIR)
+gettext.textdomain(APP)
+_ = gettext.gettext
+
+os.umask(0o022)
+
 # Used as a decorator to run things in the background
 def async(func):
     def wrapper(*args, **kwargs):
@@ -55,7 +64,7 @@ def remove_repository_via_cli(line, codename, forceYes):
             ppa_info = get_ppa_info_from_lp(user, ppa_name, codename)
             print(_("You are about to remove the following PPA:"))
             if ppa_info["description"] is not None:
-                print(" %s" % (ppa_info["description"].encode("utf-8") or ""))
+                print(" %s" % (ppa_info["description"]))
             print(_(" More info: %s") % str(ppa_info["web_link"]))
 
             if sys.stdin.isatty():
@@ -67,11 +76,11 @@ def remove_repository_via_cli(line, codename, forceYes):
                         print(_("Unable to prompt for response.  Please run with -y"))
                         sys.exit(1)
 
-        except KeyboardInterrupt, detail:
-            print _("Cancelling...")
+        except KeyboardInterrupt as detail:
+            print (_("Cancelling..."))
             sys.exit(1)
-        except Exception, detail:
-            print _("Cannot get info about PPA: '%s'.") % detail
+        except Exception as detail:
+            print (_("Cannot get info about PPA: '%s'.") % detail)
 
         (deb_line, file) = expand_ppa_line(line.strip(), codename)
         deb_line = expand_http_line(deb_line, codename)
@@ -90,8 +99,8 @@ def remove_repository_via_cli(line, codename, forceYes):
             # If file no longer contains any "deb" instances, delete it as well
             if "deb " not in content:
                 os.unlink(file)
-        except IOError, detail:
-            print _("failed to remove PPA: '%s'") % detail
+        except IOError as detail:
+            print (_("failed to remove PPA: '%s'") % detail)
 
     elif line.startswith("deb ") | line.startswith("http"):
         # Remove the repository from sources.list.d
@@ -107,8 +116,8 @@ def remove_repository_via_cli(line, codename, forceYes):
             # If file no longer contains any "deb" instances, delete it as well
             if "deb " not in content:
                 os.unlink(file)
-        except IOError, detail:
-            print _("failed to remove repository: '%s'") % detail
+        except IOError as detail:
+            print (_("failed to remove repository: '%s'") % detail)
 
 
 def add_repository_via_cli(line, codename, forceYes, use_ppas):
@@ -121,8 +130,8 @@ def add_repository_via_cli(line, codename, forceYes, use_ppas):
         ppa_name = ppa_name or "ppa"
         try:
             ppa_info = get_ppa_info_from_lp(user, ppa_name, codename)
-        except Exception, detail:
-            print _("Cannot add PPA: '%s'.") % detail
+        except Exception as detail:
+            print (_("Cannot add PPA: '%s'.") % detail)
             sys.exit(1)
 
         if "private" in ppa_info and ppa_info["private"]:
@@ -131,7 +140,7 @@ def add_repository_via_cli(line, codename, forceYes, use_ppas):
 
         print(_("You are about to add the following PPA:"))
         if ppa_info["description"] is not None:
-            print(" %s" % (ppa_info["description"].encode("utf-8") or ""))
+            print(" %s" % (ppa_info["description"]))
         print(_(" More info: %s") % str(ppa_info["web_link"]))
 
         if sys.stdin.isatty():
@@ -169,31 +178,20 @@ def get_ppa_info_from_lp(owner_name, ppa_name, base_codename):
 
     lp_url = LAUNCHPAD_PPA_API % (owner_name, ppa_name)
     try:
-        callback = CurlCallback()
-        curl = pycurl.Curl()
-        curl.setopt(pycurl.SSL_VERIFYPEER, 1)
-        curl.setopt(pycurl.SSL_VERIFYHOST, 2)
-        curl.setopt(pycurl.WRITEFUNCTION, callback.body_callback)
-        if LAUNCHPAD_PPA_CERT:
-            curl.setopt(pycurl.CAINFO, LAUNCHPAD_PPA_CERT)
-        curl.setopt(pycurl.URL, str(lp_url))
-        curl.setopt(pycurl.HTTPHEADER, ["Accept: application/json"])
-        curl.perform()
-        curl.close()
-        json_data = callback.contents
+        json_data = requests.get(lp_url).json()
     except pycurl.error as e:
         raise PPAException("Error reading %s: %s" % (lp_url, e), e)
 
     # Make sure the PPA supports our base release
     repo_url = "http://ppa.launchpad.net/%s/%s/ubuntu/dists/%s" % (owner_name, ppa_name, base_codename)
     try:
-        if (urllib.urlopen(repo_url).getcode() == 404):
+        if (urlopen(repo_url).getcode() == 404):
             raise PPAException(_("This PPA does not support %s") % base_codename)
     except Exception as e:
-        print e
+        print (e)
         raise PPAException(_("This PPA does not support %s") % base_codename)
 
-    return json.loads(json_data)
+    return json_data
 
 def encode(s):
     return re.sub("[^a-zA-Z0-9_-]", "_", s)
@@ -237,7 +235,7 @@ class CurlCallback:
         self.contents = ''
 
     def body_callback(self, buf):
-        self.contents = self.contents + buf
+        self.contents = self.contents + str(buf)
 
 
 class PPAException(Exception):
@@ -249,8 +247,6 @@ class PPAException(Exception):
     def __str__(self):
         return repr(self.value)
 
-gettext.install("mintsources", "/usr/share/linuxmint/locale")
-
 SPEED_PIX_WIDTH = 125
 SPEED_PIX_HEIGHT = 16
 
@@ -271,10 +267,10 @@ class Key():
         self.uid = ""
 
     def delete(self):
-        os.system("apt-key del %s" % self.pub)
+        os.system("apt-key del '%s'" % self.pub)
 
     def get_name(self):
-        return "<b>%s</b>\n<small><i>%s</i></small>" % (GObject.markup_escape_text(self.uid), GObject.markup_escape_text(self.pub))
+        return "%s\n<small>    %s</small>" % (GObject.markup_escape_text(self.uid), GObject.markup_escape_text(self.pub))
 
 class Mirror():
     def __init__(self, country_code, url, name):
@@ -339,8 +335,7 @@ class Repository():
         name = name.replace("/ubuntu", "")
         name = name.replace("/ppa", "")
         if self.line.startswith("deb-src"):
-            name = name.decode("utf8", "replace")
-            suffix = _("Sources").decode("utf8", "replace")
+            suffix = _("Sources")
             name = "%s (%s)" % (name, suffix)
         return "<b>%s</b>\n<small><i>%s</i></small>\n<small><i>%s</i></small>" % (name, self.line, self.file)
 
@@ -506,16 +501,15 @@ class MirrorSelectionDialog(object):
     def check_mirror_up_to_date(self, url):
         if (self.default_mirror_age is None or self.default_mirror_age < 2):
             # If the default server was updated recently, the age is irrelevant (it would measure the time between now and the last update)
-            #print "OK - default mirror age is not conclusive %s" % url
             return True
         mirror_timestamp = self.get_url_last_modified(url)
         if mirror_timestamp is None:
-            print "Error: Can't find the age of %s !!" % url
+            print ("Error: Can't find the age of %s !!" % url)
             return False
         mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
         mirror_age = (self.default_mirror_date - mirror_date).days
         if (mirror_age > 2):
-            print "Error: %s is out of date by %d days!" % (url, mirror_age)
+            print ("Error: %s is out of date by %d days!" % (url, mirror_age))
             return False
         else:
             # Age is fine :)
@@ -569,7 +563,7 @@ class MirrorSelectionDialog(object):
                 test_url = "%s/dists/%s/main/Contents-amd64.gz" % (url, self.codename)
             if (self.is_base or self.check_mirror_up_to_date("%s/db/version" % url)):
                 c = pycurl.Curl()
-                buff = cStringIO.StringIO()
+                buff = BytesIO()
                 c.setopt(pycurl.URL, test_url)
                 c.setopt(pycurl.CONNECTTIMEOUT, 5)
                 c.setopt(pycurl.TIMEOUT, 20)
@@ -581,8 +575,8 @@ class MirrorSelectionDialog(object):
             else:
                 # the mirror is not up to date
                 download_speed = -1
-        except Exception, error:
-            print "Error '%s' on url %s" % (error, url)
+        except Exception as error:
+            print ("Error '%s' on url %s" % (error, url))
             download_speed = 0
 
         self.show_speed_test_result(iter, download_speed)
@@ -613,10 +607,10 @@ class MirrorSelectionDialog(object):
 
         # Try to find out where we're located...
         try:
-            lookup = str(urllib.urlopen('http://geoip.ubuntu.com/lookup').read())
+            lookup = str(urlopen('http://geoip.ubuntu.com/lookup').read())
             cur_country_code = re.search('<CountryCode>(.*)</CountryCode>', lookup).group(1)
             if cur_country_code == 'None': cur_country_code = None
-        except Exception, detail:
+        except Exception as detail:
             cur_country_code = None  # no internet connection
 
         self.local_country_code = cur_country_code or os.environ.get('LANG', 'US').split('.')[0].split('_')[-1]  # fallback to LANG location or 'US'
@@ -684,7 +678,6 @@ class MirrorSelectionDialog(object):
             self.default_mirror_date = datetime.datetime.fromtimestamp(mirror_timestamp)
             now = datetime.datetime.now()
             self.default_mirror_age = (now - self.default_mirror_date).days
-            #print "Default mirror (%s/db/version) age: %d days" % (self.default_mirror, self.default_mirror_age)
 
         self._update_list()
         self._dialog.show_all()
@@ -708,11 +701,14 @@ class Application(object):
         # Prevent settings from being saved until the interface is fully loaded
         self._interface_loaded = False
 
-        self.lsb_codename = commands.getoutput("lsb_release -sc")
+        self.infobar_visible = False
+
+        self.lsb_codename = subprocess.getoutput("lsb_release -sc")
 
         glade_file = "/usr/lib/linuxmint/mintSources/mintSources.glade"
 
         self.builder = Gtk.Builder()
+        self.builder.set_translation_domain("mintsources")
         self.builder.add_from_file(glade_file)
         self._main_window = self.builder.get_object("main_window")
 
@@ -723,7 +719,9 @@ class Application(object):
         self._notebook = self.builder.get_object("notebook")
         self._official_repositories_box = self.builder.get_object("official_repositories_box")
 
-        config_parser = ConfigParser.RawConfigParser()
+        self.apt = mintcommon.APT(self._main_window)
+
+        config_parser = configparser.RawConfigParser()
         config_parser.read("/usr/share/mintsources/%s/mintsources.conf" % self.lsb_codename)
         self.config = {}
         self.optional_components = []
@@ -748,52 +746,8 @@ class Application(object):
         if self.config["general"]["use_ppas"] == "false":
             self.builder.get_object("vbuttonbox1").remove(self.builder.get_object("toggle_ppas"))
 
-        self.builder.get_object("reload_button_label").set_markup("%s" % _("No action required"))
-
-        self.builder.get_object("label_title_official").set_markup("%s" % _("Official repositories"))
-        self.builder.get_object("label_title_ppa").set_markup("%s" % _("PPAs"))
-        self.builder.get_object("label_title_3rd").set_markup("%s" % _("Additional repositories"))
-        self.builder.get_object("label_title_keys").set_markup("%s" % _("Authentication keys"))
-        self.builder.get_object("label_title_maintenance").set_markup("%s" % _("Maintenance"))
-
-        self.builder.get_object("label_mirrors").set_markup("<b>%s</b>" % _("Mirrors"))
         self.builder.get_object("label_mirror_description").set_markup("%s (%s)" % (_("Main"), self.config["general"]["codename"]) )
         self.builder.get_object("label_base_mirror_description").set_markup("%s (%s)" % (_("Base"), self.config["general"]["base_codename"]) )
-        self.builder.get_object("button_mirror").set_tooltip_text(_("Select a faster server..."))
-        self.builder.get_object("button_base_mirror").set_tooltip_text(_("Select a faster server..."))
-
-        self.builder.get_object("label_optional_components").set_markup("<b>%s</b>" % _("Optional components"))
-        self.builder.get_object("label_source_code").set_markup("<b>%s</b>" % _("Source code"))
-
-        self.set_button_text(self.builder.get_object("label_ppa_add"), _("Add a new PPA..."))
-        self.set_button_text(self.builder.get_object("label_ppa_edit"), _("Edit URL..."))
-        self.set_button_text(self.builder.get_object("label_ppa_remove"), _("Remove"))
-        self.set_button_text(self.builder.get_object("label_ppa_examine"), _("Open PPA"))
-        self.builder.get_object("label_ppa_examine").set_tooltip_text(_("Look inside the PPA and install packages it provides"))
-
-        self.set_button_text(self.builder.get_object("label_repository_add"), _("Add a new repository..."))
-        self.set_button_text(self.builder.get_object("label_repository_edit"), _("Edit URL..."))
-        self.set_button_text(self.builder.get_object("label_repository_remove"), _("Remove"))
-
-        self.set_button_text(self.builder.get_object("label_keys_add"), _("Import key file..."))
-        self.set_button_text(self.builder.get_object("label_keys_fetch"), _("Download a key..."))
-        self.set_button_text(self.builder.get_object("label_keys_remove"), _("Remove"))
-
-        self.builder.get_object("button_mergelist_label").set_markup("%s" % _("Fix MergeList problems"))
-        self.builder.get_object("button_mergelist").set_tooltip_text("%s" % _("If you experience MergeList problems, click this button to solve the problem."))
-        self.builder.get_object("button_purge_label").set_markup("%s" % _("Purge residual configuration"))
-        self.builder.get_object("button_purge").set_tooltip_text("%s" % _("Packages sometimes leave configuration files on the system even after they are removed."))
-        self.builder.get_object("button_remove_foreign_label").set_markup("%s" % _("Remove foreign packages"))
-        self.builder.get_object("button_remove_foreign").set_tooltip_text("%s" % _("Packages which do not come from known repositories are listed here and can be removed."))
-        self.builder.get_object("button_downgrade_foreign_label").set_markup("%s" % _("Downgrade foreign packages"))
-        self.builder.get_object("button_downgrade_foreign").set_tooltip_text("%s" % _("Packages which version does not come from known repositories are listed here and can be downgraded."))
-
-        self.builder.get_object("label_description").set_markup("<b>%s</b>" % self.config["general"]["description"])
-
-        self.builder.get_object("image_icon").set_from_icon_name("mintsources-mint", Gtk.IconSize.DND)
-
-        self.builder.get_object("source_code_cb").set_label(_("Enable source code repositories"))
-
         self.builder.get_object("source_code_cb").connect("toggled", self.apply_official_sources)
 
         self.selected_components = []
@@ -919,14 +873,12 @@ class Application(object):
         self.load_keys()
 
         if not os.path.exists("/etc/apt/sources.list.d/official-package-repositories.list"):
-            print "Sources missing, generating default sources list!"
+            print ("Sources missing, generating default sources list!")
             self.generate_missing_sources()
 
         self.detect_official_sources()
 
         self.builder.get_object("revert_button").connect("clicked", self.revert_to_default_sources)
-        self.builder.get_object("label_revert").set_markup(_("Restore the default settings"))
-        self.builder.get_object("revert_button").set_tooltip_text(_("Restore the official repositories to their default settings"))
 
         self._tab_buttons = [
             self.builder.get_object("toggle_official_repos"),
@@ -946,7 +898,6 @@ class Application(object):
 
         self.builder.get_object("button_mirror").connect("clicked", self.select_new_mirror)
         self.builder.get_object("button_base_mirror").connect("clicked", self.select_new_base_mirror)
-        self.builder.get_object("reload_button").connect("clicked", self.update_apt_cache)
 
         self.builder.get_object("button_ppa_add").connect("clicked", self.add_ppa)
         self.builder.get_object("button_ppa_edit").connect("clicked", self.edit_ppa)
@@ -1014,7 +965,7 @@ class Application(object):
         self.show_confirmation_dialog(self._main_window, _("There is no more residual configuration on the system."), image, affirmation=True)
 
     def fix_mergelist(self, widget):
-        os.system("rm /var/lib/apt/lists/* -vf")
+        os.system("rm /var/lib/apt/lists/* -vrf")
         image = Gtk.Image()
         image.set_from_icon_name("mintsources-maintenance", Gtk.IconSize.DIALOG)
         self.show_confirmation_dialog(self._main_window, _("The problem was fixed. Please reload the cache."), image, affirmation=True)
@@ -1023,20 +974,32 @@ class Application(object):
     def load_keys(self):
         self.keys = []
         key = None
-        output = commands.getoutput("apt-key list").decode("utf8", "replace")
+        output = subprocess.getoutput("apt-key list")
+        lines = []
         for line in output.split("\n"):
             line = line.strip()
-            if line.startswith("pub"):
-                pub = line[3:].strip()
-                pub = pub[6:]
-                pub = pub.split(" ")[0]
-                key = Key(pub)
-                if pub not in self.system_keys:
+            if line.startswith("/etc/apt"):
+                continue
+            if line.startswith("-----"):
+                continue
+            if line == "":
+                continue
+            lines.append(line)
+
+        for key_data in "\n".join(lines).split("pub   "):
+            key_data = key_data.split("\n")
+            if len(key_data) > 3:
+                extra = key_data[0]
+                pub = key_data[1]
+                pub_short = "".join(pub.split()[-2:])
+                name = key_data[2]
+                name = name.replace("uid ", "")
+                if "]" in name:
+                    name = name.split("]")[1].strip()
+                key = Key(pub_short)
+                key.uid = name
+                if pub_short not in self.system_keys:
                     self.keys.append(key)
-            elif line.startswith("uid") and key is not None:
-                key.uid = line[3:].strip()
-            elif line.startswith("sub") and key is not None:
-                key.sub = line[3:].strip()
 
         self._keys_model.clear()
         for key in self.keys:
@@ -1046,8 +1009,8 @@ class Application(object):
         dialog = Gtk.FileChooserDialog(_("Open.."),
                                None,
                                Gtk.FileChooserAction.OPEN,
-                               (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+                               (_("Cancel"), Gtk.ResponseType.CANCEL,
+                                _("Open"), Gtk.ResponseType.OK))
         dialog.set_default_response(Gtk.ResponseType.OK)
         response = dialog.run()
         if response == Gtk.ResponseType.OK:
@@ -1092,7 +1055,7 @@ class Application(object):
             ppa_name = ppa_name or "ppa"
             try:
                 ppa_info = get_ppa_info_from_lp(user, ppa_name, self.config["general"]["base_codename"])
-            except Exception, detail:
+            except Exception as detail:
                 self.show_error_dialog(self._main_window, _("Cannot add PPA: '%s'.") % detail)
                 return
 
@@ -1112,7 +1075,7 @@ class Application(object):
                 # Add the PPA in sources.list.d
                 with open(file, "w") as text_file:
                     text_file.write("%s\n" % deb_line)
-                    text_file.write("%s\n" % debsrc_line)
+                    text_file.write("%s\n" % "# "+debsrc_line)
 
                 # Add the package line in the UI
                 repository = Repository(self, deb_line, file, True)
@@ -1120,7 +1083,7 @@ class Application(object):
                 tree_iter = self._ppa_model.append((repository, repository.selected, repository.get_ppa_name()))
 
                 # Add the source line in the UI
-                repository = Repository(self, debsrc_line, file, True)
+                repository = Repository(self, debsrc_line, file, False)
                 self.ppas.append(repository)
                 tree_iter = self._ppa_model.append((repository, repository.selected, repository.get_ppa_name()))
 
@@ -1130,9 +1093,7 @@ class Application(object):
     def format_string(self, text):
         if text is None:
             text = ""
-        text = text.encode("utf-8")
         text = text.replace("<", "&lt;").replace(">", "&gt;")
-        text = text.decode("utf8", "replace")
         return text
 
     def edit_ppa(self, widget):
@@ -1164,8 +1125,8 @@ class Application(object):
                 ppa_name = model.get_value(iter, 2)
                 if repository.line.startswith("deb http://ppa.launchpad.net"):
                     self.builder.get_object("button_ppa_examine").set_sensitive(True)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print (detail)
 
     def on_ppa_treeview_doubleclick(self, treeview, path, column):
         self.examine_ppa(None)
@@ -1182,15 +1143,15 @@ class Application(object):
                     if line.endswith("/ubuntu"):
                         line = line[:-7]
                         ppa_owner, ppa_name = line.split("/")
-                        architecture = commands.getoutput("dpkg --print-architecture")
+                        architecture = subprocess.getoutput("dpkg --print-architecture")
                         ppa_file = "/var/lib/apt/lists/ppa.launchpad.net_%s_%s_ubuntu_dists_%s_main_binary-%s_Packages" % (ppa_owner, ppa_name, self.config["general"]["base_codename"], architecture)
                         if os.path.exists(ppa_file):
                             os.system("/usr/lib/linuxmint/mintSources/ppa_browser.py %s %s %s &" % (self.config["general"]["base_codename"], ppa_owner, ppa_name))
                         else:
-                            print "%s not found!" % ppa_file
+                            print ("%s not found!" % ppa_file)
                             self.show_error_dialog(self._main_window, _("The content of this PPA is not available. Please refresh the cache and try again."))
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print (detail)
 
     def add_repository(self, widget):
         image = Gtk.Image()
@@ -1284,8 +1245,8 @@ class Application(object):
         confirmation_button = Gtk.ResponseType.ACCEPT
         d = Gtk.Dialog(None, parent,
                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                       (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
-                      Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
+                       (_("Cancel"), Gtk.ResponseType.REJECT,
+                       _("OK"), Gtk.ResponseType.ACCEPT))
         d.set_size_request(550, 400)
         d.vbox.pack_start(s, True, True, 0)
         d.set_title("")
@@ -1334,13 +1295,15 @@ class Application(object):
 
         entry = Gtk.Entry()
         entry.set_text(default)
+        entry.set_margin_start(6)
+        entry.set_margin_end(6)
         entry.show()
         d.vbox.pack_end(entry, False, False, 0)
         entry.connect('activate', lambda _: d.response(Gtk.ResponseType.OK))
         d.set_default_response(Gtk.ResponseType.OK)
 
         r = d.run()
-        text = entry.get_text().decode('utf8')
+        text = entry.get_text()
         d.destroy()
         if r == Gtk.ResponseType.OK:
             return text
@@ -1408,22 +1371,23 @@ class Application(object):
         self.apply_official_sources()
 
     def enable_reload_button(self):
-        self.builder.get_object("reload_button").set_sensitive(True)
-        self.builder.get_object("reload_button_label").set_markup("<b>%s</b>" % _("Update the cache"))
-        self.builder.get_object("reload_button").set_tooltip_text(_("Click here to update your APT cache with your new sources"))
-        self.builder.get_object("reload_button_image").set_from_stock(Gtk.STOCK_REFRESH, Gtk.IconSize.BUTTON)
+        if not self.infobar_visible:
+            self.infobar_visible = True
+            infobar = Gtk.InfoBar()
+            infobar.set_message_type(Gtk.MessageType.INFO)
+            info_label = Gtk.Label()
+            infobar_message = "%s\n<small>%s</small>" % (_("Your configuration changed"), _("Click OK to update your APT cache"))
+            info_label.set_markup(infobar_message)
+            infobar.get_content_area().pack_start(info_label,False, False,0)
+            infobar.add_button(_("OK"), Gtk.ResponseType.OK)
+            infobar.connect("response", self._on_infobar_response)
+            self.builder.get_object("box_infobar").pack_start(infobar, True, True,0)
+            infobar.show_all()
 
-    def disable_reload_button(self):
-        self.builder.get_object("reload_button").set_sensitive(False)
-        self.builder.get_object("reload_button_label").set_markup("%s" % _("No action required"))
-        self.builder.get_object("reload_button").set_tooltip_text(_("Your APT cache is up to date"))
-        self.builder.get_object("reload_button_image").set_from_stock(Gtk.STOCK_OK, Gtk.IconSize.BUTTON)
-
-    def update_apt_cache(self, widget=None):
-        self.disable_reload_button()
-        cmd = ["sudo", "/usr/sbin/synaptic", "--hide-main-window", "--update-at-startup", "--non-interactive", "--parent-window-id", "%s" % self._main_window.get_window().get_xid()]
-        comnd = subprocess.Popen(' '.join(cmd), shell=True)
-        #returnCode = comnd.wait()
+    def _on_infobar_response(self, infobar, response_id):
+        infobar.destroy()
+        self.infobar_visible = False
+        self.apt.update_cache()
 
     def apply_official_sources(self, widget=None):
         # As long as the interface isn't fully loaded, don't save anything
@@ -1559,42 +1523,39 @@ class Application(object):
             return None
 
 if __name__ == "__main__":
-    if os.getuid() != 0:
-        os.execvp("gksu", ("", " ".join(sys.argv)))
-    else:
-        usage = "usage: %prog [options] [repository]"
-        parser = OptionParser(usage=usage)
-        #add a dummy option which can be easily ignored
-        parser.add_option("-?", dest="ignore", action="store_true", default=False)
-        parser.add_option("-y", "--yes", dest="forceYes", action="store_true",
-            help="force yes on all confirmation questions", default=False)
-        parser.add_option("-r", "--remove", dest="remove", action="store_true",
-            help="Remove the specified repository", default=False)
+    usage = "usage: %prog [options] [repository]"
+    parser = OptionParser(usage=usage)
+    #add a dummy option which can be easily ignored
+    parser.add_option("-?", dest="ignore", action="store_true", default=False)
+    parser.add_option("-y", "--yes", dest="forceYes", action="store_true",
+        help="force yes on all confirmation questions", default=False)
+    parser.add_option("-r", "--remove", dest="remove", action="store_true",
+        help="Remove the specified repository", default=False)
 
-        (options, args) = parser.parse_args()
+    (options, args) = parser.parse_args()
 
-        lsb_codename = commands.getoutput("lsb_release -sc")
-        config_dir = "/usr/share/mintsources/%s" % lsb_codename
-        if not os.path.exists(config_dir):
-            print ("LSB codename: '%s'." % lsb_codename)
-            if os.path.exists("/etc/linuxmint/info"):
-                print ("Version of base-files: '%s'." % commands.getoutput("apt version base-files"))
-                print ("Your LSB codename isn't a valid Linux Mint codename.")
-            else:
-                print ("This codename isn't currently supported.")
-            print ("Please check your LSB information with \"lsb_release -a\".")
-            sys.exit(1)
-
-        if len(args) > 1 and (args[0] == "add-apt-repository"):
-            ppa_line = args[1]
-            lsb_codename = commands.getoutput("lsb_release -sc")
-            config_parser = ConfigParser.RawConfigParser()
-            config_parser.read("/usr/share/mintsources/%s/mintsources.conf" % lsb_codename)
-            codename = config_parser.get("general", "base_codename")
-            use_ppas = config_parser.get("general", "use_ppas")
-            if options.remove:
-                remove_repository_via_cli(ppa_line, codename, options.forceYes)
-            else:
-                add_repository_via_cli(ppa_line, codename, options.forceYes, use_ppas)
+    lsb_codename = subprocess.getoutput("lsb_release -sc")
+    config_dir = "/usr/share/mintsources/%s" % lsb_codename
+    if not os.path.exists(config_dir):
+        print ("LSB codename: '%s'." % lsb_codename)
+        if os.path.exists("/etc/linuxmint/info"):
+            print ("Version of base-files: '%s'." % subprocess.getoutput("dpkg-query -f '${Version}' -W base-files"))
+            print ("Your LSB codename isn't a valid Linux Mint codename.")
         else:
-            Application().run()
+            print ("This codename isn't currently supported.")
+        print ("Please check your LSB information with \"lsb_release -a\".")
+        sys.exit(1)
+
+    if len(args) > 1 and (args[0] == "add-apt-repository"):
+        ppa_line = args[1]
+        lsb_codename = subprocess.getoutput("lsb_release -sc")
+        config_parser = configparser.RawConfigParser()
+        config_parser.read("/usr/share/mintsources/%s/mintsources.conf" % lsb_codename)
+        codename = config_parser.get("general", "base_codename")
+        use_ppas = config_parser.get("general", "use_ppas")
+        if options.remove:
+            remove_repository_via_cli(ppa_line, codename, options.forceYes)
+        else:
+            add_repository_via_cli(ppa_line, codename, options.forceYes, use_ppas)
+    else:
+        Application().run()
